//
//  Servicer.m
//  NP Checker
//
//  Created by Sven-S. Porst on Tue Feb 26 2002.
//  Copyright (c) 2001 earthlingsoft. All rights reserved.
//

#import "Servicer.h"


@implementation Servicer
- (void)NPService:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error
{
	NSString * s = [self getNP];
	NSString * returnFormat = [[NSUserDefaults standardUserDefaults] stringForKey:NPCFormatDefaultKey];
	NSString * returnString;
	NSArray * stringArray;

	//NSLog(s);
	
	if ([s isEqual:@"not playing\n"]) {
		NSBeep();
		* error = NSLocalizedString(@"iTunes is currently paused.",@"iTunes is currently paused.");
	}
	else if ([s isEqual:@"not running\n"]) {
		NSBeep();
		* error = NSLocalizedString(@"iTunes isn't running.",@"iTunes isn't running.");
	}
	else {
		stringArray = [s componentsSeparatedByString:NPCComponentSeparator];
		//NSLog([stringArray description]);
		// are all the necessary strings there?
		if ([stringArray count] < 5) {
			NSBeep();
			*error = NSLocalizedString(@"The script didn't return a properly formatted result", @"The script didn't return a properly formatted result");
			return;
		}

		// we are in business. Now check whether the format is set in user defaults - if it isn't replace with default format
		if (!returnFormat) {
			returnFormat = NPCDefaultReturnFormat;
		}

	//	NSLog(returnFormat);
		
		returnString = [NSString stringWithFormat:returnFormat, [stringArray objectAtIndex:0], [stringArray objectAtIndex:1], [stringArray objectAtIndex:2], [stringArray objectAtIndex:3], [stringArray objectAtIndex:4]];
		
		[pboard declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:nil];
		[pboard setString:returnString forType:NSStringPboardType];
	}

	// can't quit here as the pboard won't be delivered :(
	//[NSApp terminate:self];

	// normal notification doesn't work either as it will call the selector immediately, thus quitting the application before returning from this function
	//[[NSNotificationCenter defaultCenter] postNotificationName:NPCQuitNotification object:self];

	// so we enqueue the notification manually ... with a delay
	[[NSNotificationQueue defaultQueue] enqueueNotification:[NSNotification notificationWithName:NPCQuitNotification object:self] postingStyle:NSPostWhenIdle];
}



//
// run the applescript and return result
//
- (NSString*) getNP
{
	NSTask * myTask = [[[NSTask alloc] init] autorelease];
	NSPipe *	resultPipe = [NSPipe pipe];

	[myTask setStandardOutput:resultPipe];
	[myTask setLaunchPath:@"/usr/bin/osascript"];
	[myTask setArguments:[NSArray arrayWithObject:[self pathToScript]]];

	[myTask launch];
	[myTask waitUntilExit];

	return [[[NSString alloc] initWithData:[[resultPipe fileHandleForReading] readDataToEndOfFile] encoding:NSUTF8StringEncoding] autorelease];
}



//
// determine (sort of) path to script
//
- (NSString*) pathToScript
{
	NSString * scriptPath = [[NSUserDefaults standardUserDefaults] stringForKey:NPCScriptPathDefaultKey];

	//NSLog(scriptPath);
	
	if (scriptPath) 	{
		// the scriptPath default is set
		scriptPath = [scriptPath stringByStandardizingPath];
		if ([[NSFileManager defaultManager] fileExistsAtPath:scriptPath]) {
			// the file given in the default actually exists, so use it
			return scriptPath;
		}
	}

	// in all other cases return path to the script in our bundle
	return [NSBundle pathForResource:@"NP Getter" ofType:nil inDirectory:[[NSBundle mainBundle] bundlePath]];
}



//
//
- (void) quitMe:(NSNotification*) aNotification
{
	[NSApp terminate:self];
}


-(void) awakeFromNib
{
	[NSApp setServicesProvider:self];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(quitMe:) name:NPCQuitNotification object:self];
}

@end
